#!/bin/bash

# Note:  The coefficient files included with this release
#  do not include destriping of MODIS band 22 because it can adversely affect
#  fire detections.  Therefore they differ slightly from the files that are
#  used in NASA MODIS operations.

#---------------  MODIS DESTRIPING RUN SCRIPT --------------------------------
#                      21 September  2017
#
#  Significant revisions based upon a re-evaluation of the behavior
#  of the Terra instrument over its lifetime.  No changes have been
#  made to the destriping source code, but there are now 24 
#  Terra coefficient files.
#
#---------------  MODIS DESTRIPING RUN SCRIPT --------------------------------
#                      19 September  2016
#
#  Updated version 4 and verion 5 of Terra coefficient files.  Version 5 added
#  as a consequence of the Terra Safe Hold event that occurred in February 2016.
#
#
#---------------  MODIS DESTRIPING RUN SCRIPT --------------------------------
#                      12 August 2011
# Run the IMAPP MODIS destriping software on a 1KM MODIS data set.
# This version of the MODIS L1B destriping algorithm is based on the IDL version
# that has been running at CIMSS on direct broadcast since June 2003. The
# algorithm is reliable and fast in the IDL implementation.
#
# The algorithm is based on Weinreb et al., 1989: "Destriping GOES Images by
# Matching Empirical Distribution Functions". Remote Sens. Environ., 29,
# 185-195.
#
# 1. Algorithm Details
# - Accounts for both detector-to-detector and mirror side striping.
# - MODIS is treated as a 20 detector instrument in the thermal emissive bands
#  (10 detectors on each mirror side).
# - The empirical distribution function (EDF) is computed for each detector
#  (cumulative histogram of relative frequency).
# - The EDF for each detector is adjusted to match the EDF of a reference
#   in-family detector.
# - Algorithm operates on L1B scaled integers (0-32767).
# - Median scaled integer value for each band is restored following destriping.
#
# 2. Terra and Aqua MODIS Implementation in FORTRAN-90
# - Correction LUT is created for each individual granule on the fly.
# - Uncorrected scaled integers are replaced with corrected scaled integers.
# - Impact on bands 31 and 32 is equivocal.
# - For Terra MODIS, noisy detectors in some bands are replaced with neighbors.
# - For Aqua MODIS, no detectors are replaced.
# - Requires less than 60 seconds to run for each granule.
# - Text configuration file (one each for Terra and Aqua) defines
#   (a) which bands will be destriped
#   (b) which detector will be the reference for each band
#   (c) which detectors will be replaced for each band
#
# 3. Impact on the L1B 1KM data
#
# - For most of the thermal infrared bands, the impact is positive. In some it is
#   has become crucial with the update to how bad detectors are handled by the
#   the L1B software. Starting with Collect 6 L1B, bad detectors observations
#   will no longer be interpolated using adjacent detectors, but will be flagged
#   with Fill Values.  This affects Aqua Band 29 detector 6 and Aqua Band 36 
#   and therefore affects the cloud mask, cloud phase and cloud top properties
#   products. 
#
# - On most other thermal bands, the striping noise is significantly reduced, and 
#   this has a positive impact on downstream products such as the cloudmask, 
#   atmospheric profiles, and cloud top properties.
#
# - For bands 31 and 32, the impact is equivocal. However, for cloud top phase
#   computations involving differences between bands 31, 32, and 29, the impact
#   appears to be positive.
# - NOTE: The L1B 1KM file is irreversibly changed by this algorithm. It is
#   possible, but complicated, to implement this algorithm in such a way that the
#   destriping is reversible.
#
#
#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Set environmental variables
#  These variables must be set for correct execution of the destriping software
#export DESTRIPE_ROOT=${HOME}/destripe
export DESTRIPE_LUT=$DESTRIPE_ROOT/coeff
export DESTRIPE_BIN=$DESTRIPE_ROOT/bin

# SETUP AND CHECK ARGUMENTS
#-------------------------------------------------------------------------------

# Check arguments
if [ $# != 2 ]; then
  echo "Usage: run_modis_destripe.bash SAT FIL1KM"
  echo "where"
  echo "SAT    is the satellite name (terra or aqua)"
  echo "FIL1KM is the Level 1B 1000 meter radiance HDF file"
  exit -1
fi

# Extract file names
SAT=$1
FIL1KM=`basename $2`
DIR1KM=`dirname  $2`

# Get platform header (MOD or MYD)
if [ $SAT == "terra" ] || [ $SAT == "Terra" ] || [ $SAT == "TERRA" ]; then
  HEADER="MOD"
  SAT="terra"
elif [ $SAT == "aqua" ] || [ $SAT == "Aqua" ] || [ $SAT == "AQUA" ]; then
  HEADER="MYD"
  SAT="aqua"
else
  echo "Satellite name not recognized: "$SAT
  exit 1
fi

# Get date and time string from L1B 1KM meter file
DATE_TIME=`echo $FIL1KM:t | cut -d. -f2,3`

DATA_DATE=`echo $FIL1KM:t | cut -d. -f2`

FILETYPE=`echo $FIL1KM | cut -c1-3`
if [[ $FILETYPE == "MOD" ]] || [[ $FILETYPE == "MYD" ]]; then
   DATA_YEAR=`echo $DATA_DATE | cut -c2-5`
   JDAY1=`echo $DATA_DATE | cut -c6-8`
elif [[ $FILETYPE == "t1." ]] || [[ $FILETYPE == "a1." ]]; then
   DATA_YEAR=20`echo $DATA_DATE | cut -c1-2`
   JDAY1=`echo $DATA_DATE | cut -c3-5`
fi

DATA_JDAY=`expr $JDAY1  -  1`
DATA_MONTH=`date --date="$DATA_YEAR-01-01 + $DATA_JDAY days" "+%m"`
DATA_DAY=`date --date="$DATA_YEAR-01-01 + $DATA_JDAY days" "+%d"`
EPOCH_TIME=`date --date="$DATA_YEAR-$DATA_MONTH-$DATA_DAY" "+%s"` 
EPOCH_DATE=`expr $EPOCH_TIME/86400 | bc`


# Logistics to figure out which coefficient file to use
if [ $SAT == "aqua" ]; then
   TIME_V2=`date --date="2005-01-10" "+%s"`
   DATA_V2=`expr $TIME_V2/86400 | bc`

   if [ $EPOCH_DATE -lt $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v1"
   else
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v2"
   fi

elif [ $SAT == "terra" ]; then

   TIME_V1=`date --date="2000-02-24" "+%s"`
   DATA_V1=`expr $TIME_V1/86400 | bc`

   TIME_V2=`date --date="2001-01-19" "+%s"`
   DATA_V2=`expr $TIME_V2/86400 | bc`

   TIME_V3=`date --date="2002-03-19" "+%s"`
   DATA_V3=`expr $TIME_V3/86400 | bc`

   TIME_V4=`date --date="2003-12-16" "+%s"`
   DATA_V4=`expr $TIME_V4/86400 | bc`

   TIME_V5=`date --date="2004-04-27" "+%s"`
   DATA_V5=`expr $TIME_V5/86400 | bc`

   TIME_V6=`date --date="2004-06-23" "+%s"`
   DATA_V6=`expr $TIME_V6/86400 | bc`

   TIME_V7=`date --date="2005-10-01" "+%s"`
   DATA_V7=`expr $TIME_V7/86400 | bc`

   TIME_V8=`date --date="2005-11-05" "+%s"`
   DATA_V8=`expr $TIME_V8/86400 | bc`

   TIME_V9=`date --date="2006-02-19" "+%s"`
   DATA_V9=`expr $TIME_V9/86400 | bc`

   TIME_V10=`date --date="2006-06-04" "+%s"`
   DATA_V10=`expr $TIME_V10/86400 | bc`

   TIME_V11=`date --date="2006-08-29" "+%s"`
   DATA_V11=`expr $TIME_V11/86400 | bc`

   TIME_V12=`date --date="2007-07-12" "+%s"`
   DATA_V12=`expr $TIME_V12/86400 | bc`

   TIME_V13=`date --date="2008-03-01" "+%s"`
   DATA_V13=`expr $TIME_V13/86400 | bc`

   TIME_V14=`date --date="2008-11-03" "+%s"`
   DATA_V14=`expr $TIME_V14/86400 | bc`

   TIME_V15=`date --date="2010-12-01" "+%s"`
   DATA_V15=`expr $TIME_V15/86400 | bc`

   TIME_V16=`date --date="2011-07-01" "+%s"`
   DATA_V16=`expr $TIME_V16/86400 | bc`

   TIME_V17=`date --date="2012-05-16" "+%s"`
   DATA_V17=`expr $TIME_V17/86400 | bc`

   TIME_V18=`date --date="2012-07-01" "+%s"`
   DATA_V18=`expr $TIME_V18/86400 | bc`

   TIME_V19=`date --date="2013-05-05" "+%s"`
   DATA_V19=`expr $TIME_V19/86400 | bc`

   TIME_V20=`date --date="2013-09-08" "+%s"`
   DATA_V20=`expr $TIME_V20/86400 | bc`

   TIME_V21=`date --date="2014-04-15" "+%s"`
   DATA_V21=`expr $TIME_V21/86400 | bc`

   TIME_V22=`date --date="2014-08-10" "+%s"`
   DATA_V22=`expr $TIME_V22/86400 | bc`

   TIME_V23=`date --date="2015-04-02" "+%s"`
   DATA_V23=`expr $TIME_V23/86400 | bc`

   TIME_V24=`date --date="2016-02-25" "+%s"`
   DATA_V24=`expr $TIME_V24/86400 | bc`


   if [ $EPOCH_DATE -ge $DATA_V24 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v24r2"
   elif [ $EPOCH_DATE -lt $DATA_V24 ] && [ $EPOCH_DATE -ge $DATA_V23 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v23r2"
   elif [ $EPOCH_DATE -lt $DATA_V23 ] && [ $EPOCH_DATE -ge $DATA_V22 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v22r2"
   elif [ $EPOCH_DATE -lt $DATA_V22 ] && [ $EPOCH_DATE -ge $DATA_V21 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v21r2"
   elif [ $EPOCH_DATE -lt $DATA_V21 ] && [ $EPOCH_DATE -ge $DATA_V20 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v20r2"
   elif [ $EPOCH_DATE -lt $DATA_V20 ] && [ $EPOCH_DATE -ge $DATA_V19 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v19r2"
   elif [ $EPOCH_DATE -lt $DATA_V19 ] && [ $EPOCH_DATE -ge $DATA_V18 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v18r2"
   elif [ $EPOCH_DATE -lt $DATA_V18 ] && [ $EPOCH_DATE -ge $DATA_V17 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v17r2"
   elif [ $EPOCH_DATE -lt $DATA_V17 ] && [ $EPOCH_DATE -ge $DATA_V16 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v16r2"
   elif [ $EPOCH_DATE -lt $DATA_V16 ] && [ $EPOCH_DATE -ge $DATA_V15 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v15r2"
   elif [ $EPOCH_DATE -lt $DATA_V15 ] && [ $EPOCH_DATE -ge $DATA_V14 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v14r2"
   elif [ $EPOCH_DATE -lt $DATA_V14 ] && [ $EPOCH_DATE -ge $DATA_V13 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v13r2"
   elif [ $EPOCH_DATE -lt $DATA_V13 ] && [ $EPOCH_DATE -ge $DATA_V12 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v12r2"
   elif [ $EPOCH_DATE -lt $DATA_V12 ] && [ $EPOCH_DATE -ge $DATA_V11 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v11r2"
   elif [ $EPOCH_DATE -lt $DATA_V11 ] && [ $EPOCH_DATE -ge $DATA_V10 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v10r2"
   elif [ $EPOCH_DATE -lt $DATA_V10 ] && [ $EPOCH_DATE -ge $DATA_V9 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v9r2"
   elif [ $EPOCH_DATE -lt $DATA_V9 ] && [ $EPOCH_DATE -ge $DATA_V8 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v8r2"
   elif [ $EPOCH_DATE -lt $DATA_V8 ] && [ $EPOCH_DATE -ge $DATA_V7 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v7r2"
   elif [ $EPOCH_DATE -lt $DATA_V7 ] && [ $EPOCH_DATE -ge $DATA_V6 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v6r2"
   elif [ $EPOCH_DATE -lt $DATA_V6 ] && [ $EPOCH_DATE -ge $DATA_V5 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v5r2"
   elif [ $EPOCH_DATE -lt $DATA_V5 ] && [ $EPOCH_DATE -ge $DATA_V4 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v4r2"
   elif [ $EPOCH_DATE -lt $DATA_V4 ] && [ $EPOCH_DATE -ge $DATA_V3 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v3r2"
   elif [ $EPOCH_DATE -lt $DATA_V3 ] && [ $EPOCH_DATE -ge $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v2r2"
   elif [ $EPOCH_DATE -lt $DATA_V2 ]; then
      DESTRIPE_COEFF="destripe_config_${SAT}.dat.v1r2"
   fi

else
     echo "Satellite does not match aqua or terra " $SAT
     exit 1
fi

echo "Using coefficient file: " $DESTRIPE_COEFF

#-------------------------------------------------------------------------------
# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started MODIS destriping process at "`date`

# Run the destriping algorithm
${DESTRIPE_BIN}/MOD_PRDS_DB.exe ${DIR1KM}/$FIL1KM ${DESTRIPE_LUT}/$DESTRIPE_COEFF

if [ $? == 0 ]; then
   echo "File successfully destriped" 
else
   echo "Destriping Failed"
   exit 1
fi

# Print finish message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished MODIS destriping algorithm at "`date`
#-------------------------------------------------------------------------------

# Exit gracefully
exit 0
